/* GVFS - GNOME virtual filesystem layer
 *
 * Copyright (C) 2008 Alexandre Rostovtsev
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Alexandre Rostovtsev <tetromino@gmail.com>
 */

#include <config.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include <glib.h>
#include <glib/gi18n.h>
#include <gio/gio.h>
#include "gvfsjobmove.h"
#include "gvfsdaemonprotocol.h"
#include "gvfsjobsetattribute.h"
#include "gvfsjobsetattributesfrominfo.h"

G_DEFINE_TYPE (GVfsJobSetAttributesFromInfo, g_vfs_job_set_attributes_from_info, G_VFS_TYPE_JOB_DBUS)

typedef struct _GVfsJobSetAttributesFromInfoPrivate GVfsJobSetAttributesFromInfoPrivate;
struct _GVfsJobSetAttributesFromInfoPrivate {
    gboolean dispose_has_run;
};

#define G_VFS_JOB_SET_ATTRIBUTES_FROM_INFO_GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), G_VFS_TYPE_JOB_SET_ATTRIBUTES_FROM_INFO, GVfsJobSetAttributesFromInfoPrivate))

static void         run          (GVfsJob        *job);
static gboolean     try          (GVfsJob        *job);
static void         create_reply (GVfsJob               *job,
                                  GVfsDBusMount         *object,
                                  GDBusMethodInvocation *invocation);

static void
g_vfs_job_set_attributes_from_info_dispose (GObject *object)
{
  GVfsJobSetAttributesFromInfo *job;
  GVfsJobSetAttributesFromInfoPrivate *priv;

  job = G_VFS_JOB_SET_ATTRIBUTES_FROM_INFO (object);
  priv = G_VFS_JOB_SET_ATTRIBUTES_FROM_INFO_GET_PRIVATE (object);

  if (priv->dispose_has_run)
    return;

  priv->dispose_has_run = TRUE;

  g_object_unref (job->info);

  if (G_OBJECT_CLASS (g_vfs_job_set_attributes_from_info_parent_class)->dispose)
    (*G_OBJECT_CLASS (g_vfs_job_set_attributes_from_info_parent_class)->dispose) (object);
}

static void
g_vfs_job_set_attributes_from_info_finalize (GObject *object)
{
  GVfsJobSetAttributesFromInfo *job;

  job = G_VFS_JOB_SET_ATTRIBUTES_FROM_INFO (object);

  g_free (job->filename);

  if (G_OBJECT_CLASS (g_vfs_job_set_attributes_from_info_parent_class)->finalize)
    (*G_OBJECT_CLASS (g_vfs_job_set_attributes_from_info_parent_class)->finalize) (object);
}

static void
g_vfs_job_set_attributes_from_info_class_init (GVfsJobSetAttributesFromInfoClass *klass)
{
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
  GVfsJobClass *job_class = G_VFS_JOB_CLASS (klass);
  GVfsJobDBusClass *job_dbus_class = G_VFS_JOB_DBUS_CLASS (klass);

  gobject_class->dispose = g_vfs_job_set_attributes_from_info_dispose;
  gobject_class->finalize = g_vfs_job_set_attributes_from_info_finalize;
  job_class->run = run;
  job_class->try = try;
  job_dbus_class->create_reply = create_reply;

  g_type_class_add_private (klass, sizeof (GVfsJobSetAttributesFromInfoPrivate));
}

static void
g_vfs_job_set_attributes_from_info_init (GVfsJobSetAttributesFromInfo *job)
{
  GVfsJobSetAttributesFromInfoPrivate *priv;

  job->info = NULL;
  priv = G_VFS_JOB_SET_ATTRIBUTES_FROM_INFO_GET_PRIVATE (job);
  priv->dispose_has_run = FALSE;
}

gboolean
g_vfs_job_set_attributes_from_info_new_handle (GVfsDBusMount *object,
					       GDBusMethodInvocation *invocation,
					       const char *arg_path_data,
					       guint arg_flags,
					       GVariant *arg_info,
					       GVfsBackend *backend)
{
  GVfsJobSetAttributesFromInfo *job;
  GFileInfo *info = NULL;
  
  if (g_vfs_backend_invocation_first_handler (object, invocation, backend))
    return TRUE;

  if ( !(info = _g_dbus_get_file_info (arg_info, NULL)) )
    {
      g_dbus_method_invocation_return_error_literal (invocation,
                                                     G_IO_ERROR,
                                                     G_IO_ERROR_INVALID_ARGUMENT,
                                                     _("Invalid dbus message"));
      return TRUE;
    }

  job = g_object_new (G_VFS_TYPE_JOB_SET_ATTRIBUTES_FROM_INFO,
                      "object", object,
                      "invocation", invocation,
                      NULL);

  job->backend = backend;
  job->filename = g_strdup (arg_path_data);
  job->info = info;
  job->flags = arg_flags;

  g_vfs_job_source_new_job (G_VFS_JOB_SOURCE (backend), G_VFS_JOB (job));
  g_object_unref (job);

  return TRUE;
}

static void
run (GVfsJob *job)
{
  GVfsJobSetAttributesFromInfo *op_job = G_VFS_JOB_SET_ATTRIBUTES_FROM_INFO (job);
  GVfsBackendClass *class = G_VFS_BACKEND_GET_CLASS (op_job->backend);

  if (class->set_attributes_from_info == NULL)
    {
      /* TODO : try to fallback using class->set_attribute */
      g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
	               _("Operation not supported by backend"));
      return;
    }
  class->set_attributes_from_info (op_job->backend, op_job, op_job->filename, op_job->info, op_job->flags);
}

static gboolean
try (GVfsJob *job)
{
  GVfsJobSetAttributesFromInfo *op_job = G_VFS_JOB_SET_ATTRIBUTES_FROM_INFO (job);
  GVfsBackendClass *class = G_VFS_BACKEND_GET_CLASS (op_job->backend);

  if (class->try_set_attributes_from_info == NULL)
    {
      /* TODO : try to fallback using class->try_set_attribute */
      return FALSE;
    }

  return class->try_set_attributes_from_info (op_job->backend, op_job, op_job->filename, op_job->info, op_job->flags);
}

/* Might be called on an i/o thread */
static void
create_reply (GVfsJob *job,
	      GVfsDBusMount *object,
	      GDBusMethodInvocation *invocation)
{
    gvfs_dbus_mount_complete_set_attributes_from_info (object, invocation);
}
